@@ -29,6 +29,7 @@
 #include <memory>
 
 #include "zim_types.h"
+#include "zim/error.h"
 
 namespace zim
 {
@@ -38,32 +39,40 @@ namespace zim
   class Cluster : public std::enable_shared_from_this<Cluster> {
       typedef std::vector<offset_t> Offsets;
 
+    public:
       const CompressionType compression;
       const bool isExtended;
-      Offsets offsets;
+
+    private:
       std::shared_ptr<const Reader> reader;
+
+      // offset of the first blob of this cluster relative to the beginning
+      // of the (uncompressed) cluster data
       offset_t startOffset;
 
+      // offsets of the blob boundaries relative to the start of the first
+      // blob in this cluster. For a cluster with N blobs, this collection
+      // contains N+1 entries - the start of the first blob (which is always
+      // 0) and the end of the last blob are also included.
+      Offsets offsets;
+
       template<typename OFFSET_TYPE>
       offset_t read_header();
 
     public:
       Cluster(std::shared_ptr<const Reader> reader, CompressionType comp, bool isExtended);
       CompressionType getCompression() const   { return compression; }
-      bool isCompressed() const                { return compression == zimcompZip || compression == zimcompBzip2 || compression == zimcompLzma; }
+      bool isCompressed() const                { return compression != zimcompDefault && compression != zimcompNone; }
 
       blob_index_t count() const               { return blob_index_t(offsets.size() - 1); }
-      zsize_t size() const;
 
-      zsize_t getBlobSize(blob_index_t n) const  { return zsize_t(offsets[blob_index_type(n)+1].v
-                                                                - offsets[blob_index_type(n)].v); }
+      zsize_t getBlobSize(blob_index_t n) const;
+
       offset_t getBlobOffset(blob_index_t n) const { return startOffset + offsets[blob_index_type(n)]; }
       Blob getBlob(blob_index_t n) const;
       Blob getBlob(blob_index_t n, offset_t offset, zsize_t size) const;
-      void clear();
 
-      void init_from_buffer(Buffer& buffer);
-      static zsize_t read_size(const Reader* reader, bool isExtended, offset_t offset);
+      static std::shared_ptr<Cluster> read(const Reader& zimReader, offset_t clusterOffset);
   };
 
 }
