@@ -32,12 +32,24 @@
 #include "config.h"
 #include "log.h"
 #include "envvalue.h"
-#include "md5stream.h"
+#include "md5.h"
 
 log_define("zim.file.impl")
 
 namespace zim
 {
+
+namespace
+{
+
+offset_t readOffset(const Reader& reader, size_t idx)
+{
+  offset_t offset(reader.read_uint<offset_type>(offset_t(sizeof(offset_type)*idx)));
+  return offset;
+}
+
+} //unnamed namespace
+
   //////////////////////////////////////////////////////////////////////
   // FileImpl
   //
@@ -50,7 +62,6 @@ namespace zim
       direntCache(envValue("ZIM_DIRENTCACHE", DIRENT_CACHE_SIZE)),
       direntCacheLock(PTHREAD_MUTEX_INITIALIZER),
       clusterCache(envValue("ZIM_CLUSTERCACHE", CLUSTER_CACHE_SIZE)),
-      clusterCacheLock(PTHREAD_MUTEX_INITIALIZER),
       cacheUncompressedCluster(envValue("ZIM_CACHEUNCOMPRESSEDCLUSTER", false)),
       namespaceBeginLock(PTHREAD_MUTEX_INITIALIZER),
       namespaceEndLock(PTHREAD_MUTEX_INITIALIZER)
@@ -128,9 +139,14 @@ namespace zim
     }
 
     // read mime types
-    size = zsize_t(header.getUrlPtrPos() - header.getMimeListPos());
-    // No need to check access, getUrlPtrPos is in the zim file, and we are
-    // sure that getMimeListPos is 80.
+    // libzim write zims files two ways :
+    // - The old way by putting the urlPtrPos just after the mimetype.
+    // - The new way by putting the urlPtrPos at the end of the zim files.
+    //   In this case, the cluster data are always at 1024 bytes offset and we know that
+    //   mimetype list is before this.
+    // 1024 seems to be a good maximum size for the mimetype list, even for the "old" way.
+    auto endMimeList = std::min(header.getUrlPtrPos(), static_cast<zim::offset_type>(1024));
+    size = zsize_t(endMimeList - header.getMimeListPos());
     auto buffer = zimReader->get_buffer(offset_t(header.getMimeListPos()), size);
     offset_t current = offset_t(0);
     while (current.v < size.v)
@@ -144,7 +160,7 @@ namespace zim
       if (current.v + len >= size.v) {
        throw(ZimFileFormatError("Error getting mimelists."));
       }
-      
+
       std::string mimeType(buffer->data(current), len);
       mimeTypes.push_back(mimeType);
 
@@ -152,54 +168,12 @@ namespace zim
     }
   }
 
+
   std::pair<bool, article_index_t> FileImpl::findx(char ns, const std::string& url)
   {
-    log_debug("find article by url " << ns << " \"" << url << "\",  in file \"" << getFilename() << '"');
-  
-    article_index_type l = article_index_type(getNamespaceBeginOffset(ns));
-    article_index_type u = article_index_type(getNamespaceEndOffset(ns));
-  
-    if (l == u)
-    {
-      log_debug("namespace " << ns << " not found");
-      return std::pair<bool, article_index_t>(false, article_index_t(0));
-    }
-  
-    unsigned itcount = 0;
-    while (u - l > 1)
-    {
-      ++itcount;
-      article_index_type p = l + (u - l) / 2;
-      auto d = getDirent(article_index_t(p));
-  
-      int c = ns < d->getNamespace() ? -1
-            : ns > d->getNamespace() ? 1
-            : url.compare(d->getUrl());
-  
-      if (c < 0)
-        u = p;
-      else if (c > 0)
-        l = p;
-      else
-      {
-        log_debug("article found after " << itcount << " iterations in file \"" << getFilename() << "\" at index " << p);
-        return std::pair<bool, article_index_t>(true, article_index_t(p));
-      }
-    }
-
-    auto d = getDirent(article_index_t(l));
-    int c = url.compare(d->getUrl());
-  
-    if (c == 0)
-    {
-      log_debug("article found after " << itcount << " iterations in file \"" << getFilename() << "\" at index " << l);
-      return std::pair<bool, article_index_t>(true, article_index_t(l));
-    }
-  
-    log_debug("article not found after " << itcount << " iterations (\"" << d.getUrl() << "\" does not match)");
-    return std::pair<bool, article_index_t>(false, article_index_t(c < 0 ? l : u));
+    return zim::findx(*this, ns, url);
   }
-  
+
   std::pair<bool, article_index_t> FileImpl::findx(const std::string& url)
   {
     size_t start = 0;
@@ -214,27 +188,27 @@ namespace zim
   std::pair<bool, article_index_t> FileImpl::findxByTitle(char ns, const std::string& title)
   {
     log_debug("find article by title " << ns << " \"" << title << "\", in file \"" << getFilename() << '"');
-  
+
     article_index_type l = article_index_type(getNamespaceBeginOffset(ns));
     article_index_type u = article_index_type(getNamespaceEndOffset(ns));
-  
+
     if (l == u)
     {
       log_debug("namespace " << ns << " not found");
       return std::pair<bool, article_index_t>(false, article_index_t(0));
     }
-  
+
     unsigned itcount = 0;
     while (u - l > 1)
     {
       ++itcount;
       article_index_type p = l + (u - l) / 2;
       auto d = getDirentByTitle(article_index_t(p));
-  
+
       int c = ns < d->getNamespace() ? -1
             : ns > d->getNamespace() ? 1
             : title.compare(d->getTitle());
-  
+
       if (c < 0)
         u = p;
       else if (c > 0)
@@ -245,7 +219,7 @@ namespace zim
         return std::pair<bool, article_index_t>(true, article_index_t(p));
       }
     }
-  
+
     auto d = getDirentByTitle(article_index_t(l));
     int c = title.compare(d->getTitle());
 
@@ -254,12 +228,41 @@ namespace zim
       log_debug("article found after " << itcount << " iterations in file \"" << getFilename() << "\" at index " << l);
       return std::pair<bool, article_index_t>(true, article_index_t(l));
     }
-  
+
     log_debug("article not found after " << itcount << " iterations (\"" << d.getTitle() << "\" does not match)");
     return std::pair<bool, article_index_t>(false, article_index_t(c < 0 ? l : u));
   }
 
-  std::pair<FileCompound::const_iterator, FileCompound::const_iterator>
+  std::pair<bool, article_index_t> FileImpl::findxByClusterOrder(article_index_type idx)
+  {
+      std::call_once(orderOnceFlag, [this]
+      {
+          auto nb_articles = this->getCountArticles().v;
+          articleListByCluster.reserve(nb_articles);
+
+          for(zim::article_index_type i = 0; i < nb_articles; i++)
+          {
+              // This is the offset of the dirent in the zimFile
+              auto indexOffset = readOffset(*urlPtrOffsetReader, i);
+              // Get the mimeType of the dirent (offset 0) to know the type of the dirent
+              uint16_t mimeType = zimReader->read_uint<uint16_t>(indexOffset);
+              if (mimeType==Dirent::redirectMimeType || mimeType==Dirent::linktargetMimeType || mimeType == Dirent::deletedMimeType) {
+                articleListByCluster.push_back(std::make_pair(0, i));
+              } else {
+                // If it is a classic article, get the clusterNumber (at offset 8)
+                auto clusterNumber = zimReader->read_uint<zim::cluster_index_type>(indexOffset+offset_t(8));
+                articleListByCluster.push_back(std::make_pair(clusterNumber, i));
+              }
+          }
+          std::sort(articleListByCluster.begin(), articleListByCluster.end());
+      });
+
+      if (idx >= articleListByCluster.size())
+          return std::pair<bool, article_index_t>(false, article_index_t(0));
+      return std::pair<bool, article_index_t>(true, article_index_t(articleListByCluster[idx].second));
+  }
+
+  FileCompound::PartRange
   FileImpl::getFileParts(offset_t offset, zsize_t size)
   {
     return zimFile->locate(offset, size);
@@ -273,8 +276,8 @@ namespace zim
       throw ZimFileFormatError("article index out of range");
 
     pthread_mutex_lock(&direntCacheLock);
-    auto v = direntCache.getx(idx);
-    if (v.first)
+    auto v = direntCache.get(idx.v);
+    if (v.hit())
     {
       log_debug("dirent " << idx << " found in cache; hits "
                 << direntCache.getHits() << " misses "
@@ -282,7 +285,7 @@ namespace zim
                 << direntCache.hitRatio() * 100 << "% fillfactor "
                 << direntCache.fillfactor());
       pthread_mutex_unlock(&direntCacheLock);
-      return v.second;
+      return v.value();
     }
 
     log_debug("dirent " << idx << " not found in cache; hits "
@@ -291,14 +294,14 @@ namespace zim
               << direntCache.fillfactor());
     pthread_mutex_unlock(&direntCacheLock);
 
-    offset_t indexOffset = getOffset(urlPtrOffsetReader.get(), idx.v);
+    offset_t indexOffset = readOffset(*urlPtrOffsetReader, idx.v);
     // We don't know the size of the dirent because it depends of the size of
     // the title, url and extra parameters.
     // This is a pitty but we have no choices.
     // We cannot take a buffer of the size of the file, it would be really inefficient.
     // Let's do try, catch and retry while chosing a smart value for the buffer size.
     // Most dirent will be "Article" entry (header's size == 16) without extra parameters.
-    // Let's hope that url + title size will be < 256 and if not try again with a bigger size.
+    // Let's hope that url + title size will be < 256 and if not try again with a bigger size.
 
     pthread_mutex_lock(&bufferDirentLock);
     zsize_t bufferSize = zsize_t(256);
@@ -311,9 +314,9 @@ namespace zim
     while (true) {
         bufferDirentZone.reserve(size_type(bufferSize));
         zimReader->read(bufferDirentZone.data(), indexOffset, bufferSize);
-        auto direntBuffer = std::unique_ptr<Buffer>(new MemoryBuffer<false>(bufferDirentZone.data(), bufferSize));
+        const MemoryViewBuffer direntBuffer(bufferDirentZone.data(), bufferSize);
         try {
-          dirent = std::make_shared<const Dirent>(std::move(direntBuffer));
+          dirent = std::make_shared<const Dirent>(direntBuffer);
         } catch (InvalidSize&) {
           // buffer size is not enougth, try again :
           bufferSize += 256;
@@ -326,7 +329,7 @@ namespace zim
 
     log_debug("dirent read from " << indexOffset);
     pthread_mutex_lock(&direntCacheLock);
-    direntCache.put(idx, dirent);
+    direntCache.put(idx.v, dirent);
     pthread_mutex_unlock(&direntCacheLock);
 
     return dirent;
@@ -344,50 +347,30 @@ namespace zim
     if (idx >= getCountArticles())
       throw ZimFileFormatError("article index out of range");
 
-    article_index_t ret(titleIndexReader->read<article_index_type>(
+    article_index_t ret(titleIndexReader->read_uint<article_index_type>(
                             offset_t(sizeof(article_index_t)*idx.v)));
 
     return ret;
   }
 
+  FileImpl::ClusterHandle FileImpl::readCluster(cluster_index_t idx)
+  {
+    offset_t clusterOffset(getClusterOffset(idx));
+    log_debug("read cluster " << idx << " from offset " << clusterOffset);
+    return Cluster::read(*zimReader, clusterOffset);
+  }
+
   std::shared_ptr<const Cluster> FileImpl::getCluster(cluster_index_t idx)
   {
     if (idx >= getCountClusters())
       throw ZimFileFormatError("cluster index out of range");
 
-    pthread_mutex_lock(&clusterCacheLock);
-    auto cluster(clusterCache.get(idx));
-    pthread_mutex_unlock(&clusterCacheLock);
-    if (cluster)
-    {
-      log_debug("cluster " << idx << " found in cache; hits " << clusterCache.getHits() << " misses " << clusterCache.getMisses() << " ratio " << clusterCache.hitRatio() * 100 << "% fillfactor " << clusterCache.fillfactor());
-      return cluster;
-    }
-
-    offset_t clusterOffset(getClusterOffset(idx));
-    log_debug("read cluster " << idx << " from offset " << clusterOffset);
-    CompressionType comp;
-    bool extended;
-    std::shared_ptr<const Reader> reader = zimReader->sub_clusterReader(clusterOffset, &comp, &extended);
-    cluster = std::shared_ptr<Cluster>(new Cluster(reader, comp, extended));
-
-    log_debug("put cluster " << idx << " into cluster cache; hits " << clusterCache.getHits() << " misses " << clusterCache.getMisses() << " ratio " << clusterCache.hitRatio() * 100 << "% fillfactor " << clusterCache.fillfactor());
-    pthread_mutex_lock(&clusterCacheLock);
-    clusterCache.put(idx, cluster);
-    pthread_mutex_unlock(&clusterCacheLock);
-
-    return cluster;
+    return clusterCache.getOrPut(idx.v, [=](){ return readCluster(idx); });
   }
 
-  offset_t FileImpl::getOffset(const Reader* reader, size_t idx)
+  offset_t FileImpl::getClusterOffset(cluster_index_t idx) const
   {
-    offset_t offset(reader->read<offset_type>(offset_t(sizeof(offset_type)*idx)));
-    return offset;
-  }
-
-  offset_t FileImpl::getClusterOffset(cluster_index_t idx)
-  {
-    return getOffset(clusterOffsetReader.get(), idx.v);
+    return readOffset(*clusterOffsetReader, idx.v);
   }
 
   offset_t FileImpl::getBlobOffset(cluster_index_t clusterIdx, blob_index_t blobIdx)
@@ -412,20 +395,8 @@ namespace zim
     }
     pthread_mutex_unlock(&namespaceBeginLock);
 
-    article_index_type lower = 0;
-    article_index_type upper = article_index_type(getCountArticles());
-    auto d = getDirent(article_index_t(0));
-    while (upper - lower > 1)
-    {
-      article_index_type m = lower + (upper - lower) / 2;
-      auto d = getDirent(article_index_t(m));
-      if (d->getNamespace() >= ch)
-        upper = m;
-      else
-        lower = m;
-    }
+    auto ret = zim::getNamespaceBeginOffset(*this, ch);
 
-    article_index_t ret = article_index_t(d->getNamespace() < ch ? upper : lower);
     pthread_mutex_lock(&namespaceBeginLock);
     namespaceBeginCache[ch] = ret;
     pthread_mutex_unlock(&namespaceBeginLock);
@@ -447,26 +418,13 @@ namespace zim
     }
     pthread_mutex_unlock(&namespaceEndLock);
 
-    article_index_type lower = 0;
-    article_index_type upper = article_index_type(getCountArticles());
-    log_debug("namespace " << ch << " lower=" << lower << " upper=" << upper);
-    while (upper - lower > 1)
-    {
-      article_index_type m = lower + (upper - lower) / 2;
-      auto d = getDirent(article_index_t(m));
-      if (d->getNamespace() > ch)
-        upper = m;
-      else
-        lower = m;
-      log_debug("namespace " << d->getNamespace() << " m=" << m << " lower=" << lower << " upper=" << upper);
-    }
+    auto ret = zim::getNamespaceEndOffset(*this, ch);
 
     pthread_mutex_lock(&namespaceEndLock);
-    namespaceEndCache[ch] = article_index_t(upper);
+    namespaceEndCache[ch] = ret;
     pthread_mutex_unlock(&namespaceEndLock);
 
-    return article_index_t(upper);
-
+    return ret;
   }
 
   std::string FileImpl::getNamespaces()
@@ -531,7 +489,8 @@ namespace zim
     if (!header.hasChecksum())
       return false;
 
-    Md5stream md5;
+    struct zim_MD5_CTX md5ctx;
+    zim_MD5Init(&md5ctx);
 
     offset_type checksumPos = header.getChecksumPos();
     offset_type currentPos = 0;
@@ -541,7 +500,7 @@ namespace zim
       std::ifstream stream(part->second->filename());
       char ch;
       for(/*NOTHING*/ ; currentPos < checksumPos && stream.get(ch).good(); currentPos++) {
-        md5 << ch;
+        zim_MD5Update(&md5ctx, reinterpret_cast<const uint8_t*>(&ch), 1);
       }
       if (stream.bad()) {
         perror("error while reading file");
@@ -555,12 +514,11 @@ namespace zim
     if (currentPos != checksumPos) {
       return false;
     }
-           
 
     unsigned char chksumCalc[16];
     auto chksumFile = zimReader->get_buffer(offset_t(header.getChecksumPos()), zsize_t(16));
 
-    md5.getDigest(chksumCalc);
+    zim_MD5Final(chksumCalc, &md5ctx);
     if (std::memcmp(chksumFile->data(), chksumCalc, 16) != 0)
     {
       return false;
